/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.name.container;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationship;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.taxeditor.editor.name.TaxonNameEditor;

/**
 * <p>ContainerFactory class.</p>
 *
 * @author n.hoffmann
 * @version $Id: $
 */
public class ContainerFactory {
		
	/**
	 * @param taxonNameEditor
	 */
	public static void createOrUpdateAcceptedTaxonsHomotypicGroup(
			TaxonNameEditor taxonNameEditor) {

		if(taxonNameEditor.getAcceptedGroup() == null){
			taxonNameEditor.setAcceptedGroup(new AcceptedGroup(taxonNameEditor, taxonNameEditor.getTaxon().getHomotypicGroup()));
		}
		else{
			taxonNameEditor.getAcceptedGroup().redraw(taxonNameEditor.getTaxon().getHomotypicGroup());
		}
		
	}

	/**
	 * @param taxonNameEditor
	 */
	public static void createOrUpdateHeterotypicSynonymyGroups(
			TaxonNameEditor taxonNameEditor) {
		List<HomotypicalSynonymGroup> retainedGroups = new ArrayList<HomotypicalSynonymGroup>();
		
		List<HomotypicalSynonymGroup> heterotypicSynonymGroups = taxonNameEditor.getHeterotypicSynonymGroups();
		
		if (heterotypicSynonymGroups != null) {
		
			for(HomotypicalSynonymGroup group : heterotypicSynonymGroups){
				retainedGroups.add(group);
			}
		}
		
		for(HomotypicalGroup homotypicalGroup : taxonNameEditor.getTaxon().getHeterotypicSynonymyGroups()){
			HomotypicalSynonymGroup group = createOrUpdateHeterotypicSynonymyGroup(taxonNameEditor, homotypicalGroup);
			
			retainedGroups.remove(group);
		}
		
		for(HomotypicalSynonymGroup retainedGroup : retainedGroups){
			taxonNameEditor.removeGroup(retainedGroup);
		}
	}


	public static HomotypicalSynonymGroup createOrUpdateHeterotypicSynonymyGroup(
			TaxonNameEditor taxonNameEditor, HomotypicalGroup homotypicalGroup){
		HomotypicalSynonymGroup group = null;
		if(taxonNameEditor.getHomotypicalGroupContainer(homotypicalGroup) == null){
			group = new HomotypicalSynonymGroup(taxonNameEditor, homotypicalGroup);
			taxonNameEditor.addHeterotypicSynonymGroup(group);
			if(taxonNameEditor.getMisappliedGroup() != null){
				group.getControl().moveAbove(taxonNameEditor.getMisappliedGroup().getControl());
			}
		}
		else{
			group = taxonNameEditor.getHomotypicalGroupContainer(homotypicalGroup);
			group.redraw(homotypicalGroup);
		}
		
		return group;
	}

	/**
	 * @param taxonNameEditor
	 */
	public static void createOrUpdateMisapplicationsGroup(
			TaxonNameEditor taxonNameEditor) {
		MisappliedGroup group = taxonNameEditor.getMisappliedGroup();
		Taxon taxon = HibernateProxyHelper.deproxy(taxonNameEditor.getTaxon(), Taxon.class);
		if(taxon.getMisappliedNames().isEmpty()){
			taxonNameEditor.removeGroup(group);		
			taxonNameEditor.setMisapplicationsGroup(null);
		}else{
			if(group == null){
				taxonNameEditor.setMisapplicationsGroup(new MisappliedGroup(taxonNameEditor));
			}else{
				group.redraw();
			}
		}
		
	}

}
