// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport.transientServices;

import java.io.IOException;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.ParseException;
import org.hibernate.LockOptions;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.envers.query.criteria.AuditCriterion;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.INameService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.NameDeletionConfigurator;
import eu.etaxonomy.cdm.api.service.exception.ReferencedObjectUndeletableException;
import eu.etaxonomy.cdm.api.service.pager.Pager;
import eu.etaxonomy.cdm.api.service.search.DocumentSearchResult;
import eu.etaxonomy.cdm.api.service.search.SearchResult;
import eu.etaxonomy.cdm.common.monitor.IProgressMonitor;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.common.ISourceable;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.LSID;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.ReferencedEntityBase;
import eu.etaxonomy.cdm.model.common.RelationshipBase;
import eu.etaxonomy.cdm.model.common.RelationshipBase.Direction;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.name.HomotypicalGroup;
import eu.etaxonomy.cdm.model.name.HybridRelationship;
import eu.etaxonomy.cdm.model.name.HybridRelationshipType;
import eu.etaxonomy.cdm.model.name.NameRelationship;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.view.AuditEvent;
import eu.etaxonomy.cdm.model.view.AuditEventRecord;
import eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort;
import eu.etaxonomy.cdm.persistence.query.Grouping;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.cdm.persistence.query.OrderHint;
import eu.etaxonomy.cdm.strategy.cache.TaggedText;
import eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.merge.IMergeStrategy;

/**
 * @author pplitzner
 * @date 27.09.2013
 *
 */
public class TransientNameService implements INameService {

    private INameService defaultNameService;

    /**
     *
     */
    public TransientNameService(INameService defaultNameService) {
        this.defaultNameService = defaultNameService;
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(eu.etaxonomy.cdm.model.common.VersionableEntity, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<TaxonNameBase>> pageAuditEvents(TaxonNameBase t, Integer pageSize, Integer pageNumber, AuditEventSort sort, List<String> propertyPaths) {
        return defaultNameService.pageAuditEvents(t, pageSize, pageNumber, sort, propertyPaths);
    }

    /**
     * @param annotatedObj
     * @param status
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getAnnotations(eu.etaxonomy.cdm.model.common.AnnotatableEntity, eu.etaxonomy.cdm.model.common.MarkerType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Annotation> getAnnotations(TaxonNameBase annotatedObj, MarkerType status, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.getAnnotations(annotatedObj, status, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IService#clear()
     */
    @Override
    public void clear() {
        defaultNameService.clear();
    }

    /**
     * @param t
     * @param lockMode
     * @see eu.etaxonomy.cdm.api.service.IService#lock(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode)
     */
    @Override
    public void lock(TaxonNameBase t, LockOptions lockOptions) {
        defaultNameService.lock(t, lockOptions);
    }

    /**
     * @param t
     * @param lockMode
     * @param propertyPaths
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode, java.util.List)
     */
    @Override
    public void refresh(TaxonNameBase t, LockOptions lockOptions, List<String> propertyPaths) {
        defaultNameService.refresh(t, lockOptions, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache()
     */
    @Override
    public void updateTitleCache() {
        defaultNameService.updateTitleCache();
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getNextAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<TaxonNameBase> getNextAuditEvent(TaxonNameBase t) {
        return defaultNameService.getNextAuditEvent(t);
    }

    /**
     * @param annotatableEntity
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getMarkers(eu.etaxonomy.cdm.model.common.AnnotatableEntity, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Marker> getMarkers(TaxonNameBase annotatableEntity, Boolean technical, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.getMarkers(annotatableEntity, technical, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param stepSize
     * @param cacheStrategy
     * @param monitor
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache(java.lang.Class, java.lang.Integer, eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy, eu.etaxonomy.cdm.common.monitor.IProgressMonitor)
     */
    @Override
    public void updateTitleCache(Class<? extends TaxonNameBase> clazz, Integer stepSize, IIdentifiableEntityCacheStrategy<TaxonNameBase> cacheStrategy, IProgressMonitor monitor) {
        defaultNameService.updateTitleCache(clazz, stepSize, cacheStrategy, monitor);
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getPreviousAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<TaxonNameBase> getPreviousAuditEvent(TaxonNameBase t) {
        return defaultNameService.getPreviousAuditEvent(t);
    }

    /**
     * @param clazz
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#count(java.lang.Class)
     */
    @Override
    public int count(Class<? extends TaxonNameBase> clazz) {
        return defaultNameService.count(clazz);
    }

    /**
     * @param name
     * @param config
     * @return
     * @throws ReferencedObjectUndeletableException
     * @see eu.etaxonomy.cdm.api.service.INameService#delete(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.api.service.config.NameDeletionConfigurator)
     */
    @Override
    public DeleteResult delete(TaxonNameBase name, NameDeletionConfigurator config)  {
        return defaultNameService.delete(name, config);
    }

    /**
     * @param clazz
     * @param from
     * @param to
     * @param criteria
     * @param pageSize
     * @param pageValue
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(java.lang.Class, eu.etaxonomy.cdm.model.view.AuditEvent, eu.etaxonomy.cdm.model.view.AuditEvent, java.util.List, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<TaxonNameBase>> pageAuditEvents(Class<? extends TaxonNameBase> clazz, AuditEvent from, AuditEvent to, List<AuditCriterion> criteria, Integer pageSize, Integer pageValue, AuditEventSort sort, List<String> propertyPaths) {
        return defaultNameService.pageAuditEvents(clazz, from, to, criteria, pageSize, pageValue, sort, propertyPaths);
    }

    /**
     * @param lsid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#find(eu.etaxonomy.cdm.model.common.LSID)
     */
    @Override
    public TaxonNameBase find(LSID lsid) {
        return defaultNameService.find(lsid);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#delete(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public DeleteResult delete(TaxonNameBase persistentObject) {
        return defaultNameService.delete(persistentObject);
    }

    /**
     * @param clazz
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#groupMarkers(java.lang.Class, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public List<Object[]> groupMarkers(Class<? extends TaxonNameBase> clazz, Boolean technical, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultNameService.groupMarkers(clazz, technical, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#exists(java.util.UUID)
     */
    @Override
    public boolean exists(UUID uuid) {
        return defaultNameService.exists(uuid);
    }

    /**
     * @param name
     * @param typeDesignation
     * @see eu.etaxonomy.cdm.api.service.INameService#deleteTypeDesignation(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.TypeDesignationBase)
     */
    @Override
    public void deleteTypeDesignation(TaxonNameBase name, TypeDesignationBase typeDesignation) {
        defaultNameService.deleteTypeDesignation(name, typeDesignation);
    }

    /**
     * @param x
     * @param y
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#replace(eu.etaxonomy.cdm.model.common.IdentifiableEntity, eu.etaxonomy.cdm.model.common.IdentifiableEntity)
     */
    @Override
    public TaxonNameBase replace(TaxonNameBase x, TaxonNameBase y) {
        return defaultNameService.replace(x, y);
    }

    /**
     * @param uuidSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.Set)
     */
    @Override
    public List<TaxonNameBase> find(Set<UUID> uuidSet) {
        return defaultNameService.find(uuidSet);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSources(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<IdentifiableSource> getSources(TaxonNameBase t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultNameService.getSources(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param typeDesignationCollection
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#saveTypeDesignationAll(java.util.Collection)
     */
    @Override
    public Map<UUID, TypeDesignationBase> saveTypeDesignationAll(Collection<TypeDesignationBase> typeDesignationCollection) {
        return null;
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.UUID)
     */
    @Override
    public TaxonNameBase find(UUID uuid) {
        return defaultNameService.find(uuid);
    }

    /**
     * @param referencedEntityCollection
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#saveReferencedEntitiesAll(java.util.Collection)
     */
    @Override
    public Map<UUID, ReferencedEntityBase> saveReferencedEntitiesAll(Collection<ReferencedEntityBase> referencedEntityCollection) {
        return null;
    }

    /**
     * @param homotypicalGroups
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#saveAllHomotypicalGroups(java.util.Collection)
     */
    @Override
    public Map<UUID, HomotypicalGroup> saveAllHomotypicalGroups(Collection<HomotypicalGroup> homotypicalGroups) {
        return null;
    }

    /**
     * @param id
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(int)
     */
    @Override
    public TaxonNameBase find(int id) {
        return defaultNameService.find(id);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getRights(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<Rights> getRights(TaxonNameBase t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultNameService.getRights(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param clazz
     * @param technical
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#countMarkers(java.lang.Class, java.lang.Boolean)
     */
    @Override
    public int countMarkers(Class<? extends TaxonNameBase> clazz, Boolean technical) {
        return defaultNameService.countMarkers(clazz, technical);
    }

    /**
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getAllNomenclaturalStatus(int, int)
     */
    @Override
    public List<NomenclaturalStatus> getAllNomenclaturalStatus(int limit, int start) {
        return defaultNameService.getAllNomenclaturalStatus(limit, start);
    }

    /**
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getAllTypeDesignations(int, int)
     */
    @Override
    public List<TypeDesignationBase> getAllTypeDesignations(int limit, int start) {
        return defaultNameService.getAllTypeDesignations(limit, start);
    }

    /**
     * @param idSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#findById(java.util.Set)
     */
    @Override
    public List<TaxonNameBase> findById(Set<Integer> idSet) {
        return defaultNameService.findById(idSet);
    }

    /**
     * @param name
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getNamesByName(java.lang.String)
     */
    @Override
    public List<TaxonNameBase> getNamesByName(String name) {
        return defaultNameService.getNamesByName(name);
    }

    /**
     * @param nameCache
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getNamesByNameCache(java.lang.String)
     */
    @Override
    public List<NonViralName> getNamesByNameCache(String nameCache) {
        return defaultNameService.getNamesByNameCache(nameCache);
    }

    /**
     * @param clazz
     * @param idInSource
     * @param idNamespace
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSourcedObjectByIdInSource(java.lang.Class, java.lang.String, java.lang.String)
     */
    @Override
    public ISourceable getSourcedObjectByIdInSource(Class clazz, String idInSource, String idNamespace) {
        return defaultNameService.getSourcedObjectByIdInSource(clazz, idInSource, idNamespace);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#getSession()
     */
    @Override
    public Session getSession() {
        return defaultNameService.getSession();
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getUuidAndTitleCache()
     */
    @Override
    public List<UuidAndTitleCache<TaxonNameBase>> getUuidAndTitleCache() {
        return defaultNameService.getUuidAndTitleCache();
    }

    /**
     * @param clazz
     * @param limit
     * @param start
     * @param groups
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#group(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Object[]> group(Class<? extends TaxonNameBase> clazz, Integer limit, Integer start, List<Grouping> groups, List<String> propertyPaths) {
        return defaultNameService.group(clazz, limit, start, groups, propertyPaths);
    }

    /**
     * @param titleCache
     * @param matchMode
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findNamesByTitleCache(java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List)
     */
    @Override
    public List<NonViralName> findNamesByTitleCache(String titleCache, MatchMode matchMode, List<String> propertyPaths) {
        return defaultNameService.findNamesByTitleCache(titleCache, matchMode, propertyPaths);
    }

    /**
     * @param nameCache
     * @param matchMode
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findNamesByNameCache(java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List)
     */
    @Override
    public List<NonViralName> findNamesByNameCache(String nameCache, MatchMode matchMode, List<String> propertyPaths) {
        return defaultNameService.findNamesByNameCache(nameCache, matchMode, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<TaxonNameBase> findByTitle(Class<? extends TaxonNameBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.findByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param uuid
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findNameByUuid(java.util.UUID, java.util.List)
     */
    @Override
    public NonViralName findNameByUuid(UUID uuid, List<String> propertyPaths) {
        return defaultNameService.findNameByUuid(uuid, propertyPaths);
    }

    /**
     * @param name
     * @param sessionObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getNamesByName(java.lang.String, eu.etaxonomy.cdm.model.common.CdmBase)
     */
    @Override
    public List getNamesByName(String name, CdmBase sessionObject) {
        return defaultNameService.getNamesByName(name, sessionObject);
    }

    /**
     * @param name
     * @param accuracy
     * @param languages
     * @param highlightFragments
     * @param propertyPaths
     * @param maxNoOfResults
     * @return
     * @throws CorruptIndexException
     * @throws IOException
     * @throws ParseException
     * @see eu.etaxonomy.cdm.api.service.INameService#findByNameFuzzySearch(java.lang.String, float, java.util.List, boolean, java.util.List, int)
     */
    @Override
    public List<SearchResult<TaxonNameBase>> findByNameFuzzySearch(String name, float accuracy, List<Language> languages, boolean highlightFragments, List<String> propertyPaths, int maxNoOfResults) throws CorruptIndexException, IOException, ParseException {
        return defaultNameService.findByNameFuzzySearch(name, accuracy, languages, highlightFragments, propertyPaths, maxNoOfResults);
    }

    /**
     * @param type
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <S extends TaxonNameBase> List<S> list(Class<S> type, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.list(type, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Pager<TaxonNameBase> findByTitle(IIdentifiableEntityServiceConfigurator<TaxonNameBase> configurator) {
        return defaultNameService.findByTitle(configurator);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List)
     */
    @Override
    public Integer countByTitle(Class<? extends TaxonNameBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria) {
        return defaultNameService.countByTitle(clazz, queryString, matchmode, criteria);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID)
     */
    @Override
    public TaxonNameBase load(UUID uuid) {
        return defaultNameService.load(uuid);
    }

    /**
     * @param name
     * @param accuracy
     * @param languages
     * @param highlightFragments
     * @param maxNoOfResults
     * @return
     * @throws CorruptIndexException
     * @throws IOException
     * @throws ParseException
     * @see eu.etaxonomy.cdm.api.service.INameService#findByNameFuzzySearch(java.lang.String, float, java.util.List, boolean, int)
     */
    @Override
    public List<DocumentSearchResult> findByNameFuzzySearch(String name, float accuracy, List<Language> languages, boolean highlightFragments, int maxNoOfResults) throws CorruptIndexException, IOException, ParseException {
        return defaultNameService.findByNameFuzzySearch(name, accuracy, languages, highlightFragments, maxNoOfResults);
    }

    /**
     * @param uuid
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID, java.util.List)
     */
    @Override
    public TaxonNameBase load(UUID uuid, List<String> propertyPaths) {
        return defaultNameService.load(uuid, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Integer countByTitle(IIdentifiableEntityServiceConfigurator<TaxonNameBase> configurator) {
        return defaultNameService.countByTitle(configurator);
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#merge(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public TaxonNameBase merge(TaxonNameBase transientObject) {
        return defaultNameService.merge(transientObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<TaxonNameBase> listByTitle(Class<? extends TaxonNameBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.listByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#page(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <S extends TaxonNameBase> Pager<S> page(Class<S> type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.page(type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param accuracy
     * @param languages
     * @param highlightFragments
     * @param maxNoOfResults
     * @return
     * @throws CorruptIndexException
     * @throws IOException
     * @throws ParseException
     * @see eu.etaxonomy.cdm.api.service.INameService#findByFuzzyNameCacheSearch(java.lang.String, float, java.util.List, boolean, int)
     */
    @Override
    public List<DocumentSearchResult> findByFuzzyNameCacheSearch(String name, float accuracy, List<Language> languages, boolean highlightFragments, int maxNoOfResults) throws CorruptIndexException, IOException, ParseException {
        return defaultNameService.findByFuzzyNameCacheSearch(name, accuracy, languages, highlightFragments, maxNoOfResults);
    }

    /**
     * @param name
     * @param wildcard
     * @param languages
     * @param highlightFragments
     * @param maxNoOfResults
     * @return
     * @throws CorruptIndexException
     * @throws IOException
     * @throws ParseException
     * @see eu.etaxonomy.cdm.api.service.INameService#findByNameExactSearch(java.lang.String, boolean, java.util.List, boolean, int)
     */
    @Override
    public List<DocumentSearchResult> findByNameExactSearch(String name, boolean wildcard, List<Language> languages, boolean highlightFragments, int maxNoOfResults) throws CorruptIndexException, IOException, ParseException {
        return defaultNameService.findByNameExactSearch(name, wildcard, languages, highlightFragments, maxNoOfResults);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID refresh(TaxonNameBase persistentObject) {
        return defaultNameService.refresh(persistentObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByReferenceTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<TaxonNameBase> listByReferenceTitle(Class<? extends TaxonNameBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.listByReferenceTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param tableName
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#rows(java.lang.String, int, int)
     */
    @Override
    public List<TaxonNameBase> rows(String tableName, int limit, int start) {
        return defaultNameService.rows(tableName, limit, start);
    }

    /**
     * @param newInstances
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(java.util.Collection)
     */
    @Override
    public Map<UUID, TaxonNameBase> save(Collection<TaxonNameBase> newInstances) {
        return null;
    }

    /**
     * @param newInstance
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID save(TaxonNameBase newInstance) {
        return null;
    }

    /**
     * @param title
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findNamesByTitle(java.lang.String)
     */
    @Override
    public List findNamesByTitle(String title) {
        return defaultNameService.findNamesByTitle(title);
    }

    /**
     * @param title
     * @param sessionObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findNamesByTitle(java.lang.String, eu.etaxonomy.cdm.model.common.CdmBase)
     */
    @Override
    public List findNamesByTitle(String title, CdmBase sessionObject) {
        return defaultNameService.findNamesByTitle(title, sessionObject);
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID saveOrUpdate(TaxonNameBase transientObject) {
        return null;
    }

    /**
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getAllHomotypicalGroups(int, int)
     */
    @Override
    public List<HomotypicalGroup> getAllHomotypicalGroups(int limit, int start) {
        return defaultNameService.getAllHomotypicalGroups(limit, start);
    }

    /**
     * @param limit
     * @param start
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.INameService#getAllRelationships(int, int)
     */
    @Deprecated
    @Override
    public List<RelationshipBase> getAllRelationships(int limit, int start) {
        return defaultNameService.getAllRelationships(limit, start);
    }

    /**
     * @param transientObjects
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(java.util.Collection)
     */
    @Override
    public Map<UUID, TaxonNameBase> saveOrUpdate(Collection<TaxonNameBase> transientObjects) {
        return null;
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#update(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID update(TaxonNameBase transientObject) {
        return defaultNameService.update(transientObject);
    }

    /**
     * @param example
     * @param includeProperties
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(eu.etaxonomy.cdm.model.common.ICdmBase, java.util.Set, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<TaxonNameBase> list(TaxonNameBase example, Set<String> includeProperties, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.list(example, includeProperties, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param matchStrategy
     * @param mergeStrategy
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#deduplicate(java.lang.Class, eu.etaxonomy.cdm.strategy.match.IMatchStrategy, eu.etaxonomy.cdm.strategy.merge.IMergeStrategy)
     */
    @Override
    public int deduplicate(Class<? extends TaxonNameBase> clazz, IMatchStrategy matchStrategy, IMergeStrategy mergeStrategy) {
        return defaultNameService.deduplicate(clazz, matchStrategy, mergeStrategy);
    }

    /**
     * @param name
     * @param direction
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#listNameRelationships(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.common.RelationshipBase.Direction, eu.etaxonomy.cdm.model.name.NameRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<NameRelationship> listNameRelationships(TaxonNameBase name, Direction direction, NameRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.listNameRelationships(name, direction, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param matchMode
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findTitleCache(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, eu.etaxonomy.cdm.persistence.query.MatchMode)
     */
    @Override
    public Pager<TaxonNameBase> findTitleCache(Class<? extends TaxonNameBase> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, MatchMode matchMode) {
        return defaultNameService.findTitleCache(clazz, queryString, pageSize, pageNumber, orderHints, matchMode);
    }

    /**
     * @param name
     * @param direction
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#pageNameRelationships(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.common.RelationshipBase.Direction, eu.etaxonomy.cdm.model.name.NameRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<NameRelationship> pageNameRelationships(TaxonNameBase name, Direction direction, NameRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.pageNameRelationships(name, direction, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.INameService#listFromNameRelationships(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.NameRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Deprecated
    @Override
    public List<NameRelationship> listFromNameRelationships(TaxonNameBase name, NameRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.listFromNameRelationships(name, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.INameService#pageFromNameRelationships(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.NameRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Deprecated
    @Override
    public Pager<NameRelationship> pageFromNameRelationships(TaxonNameBase name, NameRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.pageFromNameRelationships(name, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.INameService#listToNameRelationships(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.NameRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Deprecated
    @Override
    public List<NameRelationship> listToNameRelationships(TaxonNameBase name, NameRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.listToNameRelationships(name, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @deprecated
     * @see eu.etaxonomy.cdm.api.service.INameService#pageToNameRelationships(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.NameRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Deprecated
    @Override
    public Pager<NameRelationship> pageToNameRelationships(TaxonNameBase name, NameRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.pageToNameRelationships(name, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getHybridNames(eu.etaxonomy.cdm.model.name.NonViralName, eu.etaxonomy.cdm.model.name.HybridRelationshipType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<HybridRelationship> getHybridNames(NonViralName name, HybridRelationshipType type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.getHybridNames(name, type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param name
     * @param status
     * @param pageSize
     * @param pageNumber
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getTypeDesignations(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus, java.lang.Integer, java.lang.Integer)
     */
    @Override
    public Pager<TypeDesignationBase> getTypeDesignations(TaxonNameBase name, SpecimenTypeDesignationStatus status, Integer pageSize, Integer pageNumber) {
        return defaultNameService.getTypeDesignations(name, status, pageSize, pageNumber);
    }

    /**
     * @param name
     * @param status
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getTypeDesignations(eu.etaxonomy.cdm.model.name.TaxonNameBase, eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<TypeDesignationBase> getTypeDesignations(TaxonNameBase name, SpecimenTypeDesignationStatus status, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultNameService.getTypeDesignations(name, status, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uninomial
     * @param infraGenericEpithet
     * @param specificEpithet
     * @param infraspecificEpithet
     * @param rank
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#searchNames(java.lang.String, java.lang.String, java.lang.String, java.lang.String, eu.etaxonomy.cdm.model.name.Rank, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<TaxonNameBase> searchNames(String uninomial, String infraGenericEpithet, String specificEpithet, String infraspecificEpithet, Rank rank, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.searchNames(uninomial, infraGenericEpithet, specificEpithet, infraspecificEpithet, rank, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#search(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<TaxonNameBase> search(Class<? extends TaxonNameBase> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.search(clazz, queryString, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getUuidAndTitleCacheOfNames()
     */
    @Override
    public List<UuidAndTitleCache> getUuidAndTitleCacheOfNames() {
        return defaultNameService.getUuidAndTitleCacheOfNames();
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findByName(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<TaxonNameBase> findByName(Class<? extends TaxonNameBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultNameService.findByName(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#findHomotypicalGroup(java.util.UUID)
     */
    @Override
    public HomotypicalGroup findHomotypicalGroup(UUID uuid) {
        return defaultNameService.findHomotypicalGroup(uuid);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.INameService#getTaggedName(java.util.UUID)
     */
    @Override
    public List<TaggedText> getTaggedName(UUID uuid) {
        return defaultNameService.getTaggedName(uuid);
    }

	@Override
	public List<String> isDeletable(TaxonNameBase object,
			DeleteConfiguratorBase config) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public TaxonNameBase findWithoutFlush(UUID uuid) {
		return defaultNameService.findWithoutFlush(uuid);
	}


}
