// $Id$
/**
* Copyright (C) 2013 EDIT
* European Distributed Institute of Taxonomy
* http://www.e-taxonomy.eu
*
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/
package eu.etaxonomy.taxeditor.editor.view.dataimport.transientServices;

import java.io.IOException;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.ParseException;
import org.hibernate.LockOptions;
import org.hibernate.Session;
import org.hibernate.criterion.Criterion;
import org.hibernate.envers.query.criteria.AuditCriterion;
import org.hibernate.search.spatial.impl.Rectangle;

import eu.etaxonomy.cdm.api.facade.DerivedUnitFacade;
import eu.etaxonomy.cdm.api.facade.DerivedUnitFacadeNotSupportedException;
import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.config.DeleteConfiguratorBase;
import eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.SpecimenDeleteConfigurator;
import eu.etaxonomy.cdm.api.service.dto.DerivateHierarchyDTO;
import eu.etaxonomy.cdm.api.service.pager.Pager;
import eu.etaxonomy.cdm.api.service.search.SearchResult;
import eu.etaxonomy.cdm.api.service.util.TaxonRelationshipEdge;
import eu.etaxonomy.cdm.common.monitor.IProgressMonitor;
import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.ISourceable;
import eu.etaxonomy.cdm.model.common.IdentifiableSource;
import eu.etaxonomy.cdm.model.common.LSID;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.Marker;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.UuidAndTitleCache;
import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.location.Country;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.Rights;
import eu.etaxonomy.cdm.model.molecular.DnaSample;
import eu.etaxonomy.cdm.model.molecular.Sequence;
import eu.etaxonomy.cdm.model.occurrence.DerivationEvent;
import eu.etaxonomy.cdm.model.occurrence.DerivedUnit;
import eu.etaxonomy.cdm.model.occurrence.DeterminationEvent;
import eu.etaxonomy.cdm.model.occurrence.FieldUnit;
import eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.cdm.model.taxon.TaxonBase;
import eu.etaxonomy.cdm.model.view.AuditEvent;
import eu.etaxonomy.cdm.model.view.AuditEventRecord;
import eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort;
import eu.etaxonomy.cdm.persistence.query.Grouping;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.cdm.persistence.query.OrderHint;
import eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy;
import eu.etaxonomy.cdm.strategy.match.IMatchStrategy;
import eu.etaxonomy.cdm.strategy.merge.IMergeStrategy;

/**
 * @author pplitzner
 * @date 26.09.2013
 *
 */
public class TransientOccurenceService implements IOccurrenceService {

    private final IOccurrenceService defaultService;
    private final Collection<SpecimenOrObservationBase<?>> units;

    /**
     * @param defaultService
     */
    public TransientOccurenceService(IOccurrenceService defaultService) {
        this.defaultService = defaultService;
        this.units = new HashSet<SpecimenOrObservationBase<?>>();
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(eu.etaxonomy.cdm.model.common.VersionableEntity, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<SpecimenOrObservationBase>> pageAuditEvents(SpecimenOrObservationBase t, Integer pageSize, Integer pageNumber, AuditEventSort sort, List<String> propertyPaths) {
        return defaultService.pageAuditEvents(t, pageSize, pageNumber, sort, propertyPaths);
    }

    /**
     * @param annotatedObj
     * @param status
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getAnnotations(eu.etaxonomy.cdm.model.common.AnnotatableEntity, eu.etaxonomy.cdm.model.common.MarkerType, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Annotation> getAnnotations(SpecimenOrObservationBase annotatedObj, MarkerType status, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.getAnnotations(annotatedObj, status, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IService#clear()
     */
    @Override
    public void clear() {
        defaultService.clear();
    }

    /**
     * @param t
     * @param lockMode
     * @see eu.etaxonomy.cdm.api.service.IService#lock(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode)
     */
    @Override
    public void lock(SpecimenOrObservationBase t, LockOptions lockOptions) {
        defaultService.lock(t, lockOptions);
    }

    /**
     * @param t
     * @param lockMode
     * @param propertyPaths
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase, org.hibernate.LockMode, java.util.List)
     */
    @Override
    public void refresh(SpecimenOrObservationBase t, LockOptions lockOptions, List<String> propertyPaths) {
        defaultService.refresh(t, lockOptions, propertyPaths);
    }

    /**
     *
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache()
     */
    @Override
    public void updateTitleCache() {
        defaultService.updateTitleCache();
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getNextAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<SpecimenOrObservationBase> getNextAuditEvent(SpecimenOrObservationBase t) {
        return defaultService.getNextAuditEvent(t);
    }

    /**
     * @param annotatableEntity
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#getMarkers(eu.etaxonomy.cdm.model.common.AnnotatableEntity, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<Marker> getMarkers(SpecimenOrObservationBase annotatableEntity, Boolean technical, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.getMarkers(annotatableEntity, technical, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param stepSize
     * @param cacheStrategy
     * @param monitor
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#updateTitleCache(java.lang.Class, java.lang.Integer, eu.etaxonomy.cdm.strategy.cache.common.IIdentifiableEntityCacheStrategy, eu.etaxonomy.cdm.common.monitor.IProgressMonitor)
     */
    @Override
    public void updateTitleCache(Class<? extends SpecimenOrObservationBase> clazz, Integer stepSize, IIdentifiableEntityCacheStrategy<SpecimenOrObservationBase> cacheStrategy, IProgressMonitor monitor) {
        defaultService.updateTitleCache(clazz, stepSize, cacheStrategy, monitor);
    }

    /**
     * @param t
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#getPreviousAuditEvent(eu.etaxonomy.cdm.model.common.VersionableEntity)
     */
    @Override
    public AuditEventRecord<SpecimenOrObservationBase> getPreviousAuditEvent(SpecimenOrObservationBase t) {
        return defaultService.getPreviousAuditEvent(t);
    }

    /**
     * @param iso639
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getCountryByIso(java.lang.String)
     */
    @Override
    public Country getCountryByIso(String iso639) {
        return defaultService.getCountryByIso(iso639);
    }

    /**
     * @param clazz
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#count(java.lang.Class)
     */
    @Override
    public int count(Class<? extends SpecimenOrObservationBase> clazz) {
        return defaultService.count(clazz);
    }

    /**
     * @param name
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getCountryByName(java.lang.String)
     */
    @Override
    public List<Country> getCountryByName(String name) {
        return defaultService.getCountryByName(name);
    }

    /**
     * @param type
     * @param determinedAs
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#list(java.lang.Class, eu.etaxonomy.cdm.model.taxon.TaxonBase, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<SpecimenOrObservationBase> list(Class<? extends SpecimenOrObservationBase> type, TaxonBase determinedAs, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.list(type, determinedAs, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param from
     * @param to
     * @param criteria
     * @param pageSize
     * @param pageValue
     * @param sort
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IVersionableService#pageAuditEvents(java.lang.Class, eu.etaxonomy.cdm.model.view.AuditEvent, eu.etaxonomy.cdm.model.view.AuditEvent, java.util.List, java.lang.Integer, java.lang.Integer, eu.etaxonomy.cdm.persistence.dao.common.AuditEventSort, java.util.List)
     */
    @Override
    public Pager<AuditEventRecord<SpecimenOrObservationBase>> pageAuditEvents(Class<? extends SpecimenOrObservationBase> clazz, AuditEvent from, AuditEvent to, List<AuditCriterion> criteria, Integer pageSize, Integer pageValue, AuditEventSort sort, List<String> propertyPaths) {
        return defaultService.pageAuditEvents(clazz, from, to, criteria, pageSize, pageValue, sort, propertyPaths);
    }

    /**
     * @param lsid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#find(eu.etaxonomy.cdm.model.common.LSID)
     */
    @Override
    public SpecimenOrObservationBase find(LSID lsid) {
        return defaultService.find(lsid);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#delete(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public DeleteResult delete(SpecimenOrObservationBase persistentObject) {
        return defaultService.delete(persistentObject);
    }

    /**
     * @param clazz
     * @param technical
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#groupMarkers(java.lang.Class, java.lang.Boolean, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public List<Object[]> groupMarkers(Class<? extends SpecimenOrObservationBase> clazz, Boolean technical, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.groupMarkers(clazz, technical, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#exists(java.util.UUID)
     */
    @Override
    public boolean exists(UUID uuid) {
        return defaultService.exists(uuid);
    }

    /**
     * @param x
     * @param y
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#replace(eu.etaxonomy.cdm.model.common.IdentifiableEntity, eu.etaxonomy.cdm.model.common.IdentifiableEntity)
     */
    @Override
    public SpecimenOrObservationBase replace(SpecimenOrObservationBase x, SpecimenOrObservationBase y) {
        return defaultService.replace(x, y);
    }

    /**
     * @param uuidSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.Set)
     */
    @Override
    public List<SpecimenOrObservationBase> find(Set<UUID> uuidSet) {
        return defaultService.find(uuidSet);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSources(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<IdentifiableSource> getSources(SpecimenOrObservationBase t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getSources(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(java.util.UUID)
     */
    @Override
    public SpecimenOrObservationBase find(UUID uuid) {
        return defaultService.find(uuid);
    }

    /**
     * @param id
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#find(int)
     */
    @Override
    public SpecimenOrObservationBase find(int id) {
        return defaultService.find(id);
    }

    /**
     * @param t
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getRights(eu.etaxonomy.cdm.model.common.IdentifiableEntity, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<Rights> getRights(SpecimenOrObservationBase t, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getRights(t, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param clazz
     * @param technical
     * @return
     * @see eu.etaxonomy.cdm.api.service.IAnnotatableService#countMarkers(java.lang.Class, java.lang.Boolean)
     */
    @Override
    public int countMarkers(Class<? extends SpecimenOrObservationBase> clazz, Boolean technical) {
        return defaultService.countMarkers(clazz, technical);
    }

    /**
     * @param occurence
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getMedia(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<Media> getMedia(SpecimenOrObservationBase occurence, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getMedia(occurence, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param idSet
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#findById(java.util.Set)
     */
    @Override
    public List<SpecimenOrObservationBase> findById(Set<Integer> idSet) {
        return defaultService.findById(idSet);
    }

    /**
     * @param clazz
     * @param idInSource
     * @param idNamespace
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getSourcedObjectByIdInSource(java.lang.Class, java.lang.String, java.lang.String)
     */
    @Override
    public ISourceable getSourcedObjectByIdInSource(Class clazz, String idInSource, String idNamespace) {
        return defaultService.getSourcedObjectByIdInSource(clazz, idInSource, idNamespace);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#getSession()
     */
    @Override
    public Session getSession() {
        return defaultService.getSession();
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#getUuidAndTitleCache()
     */
    @Override
    public List<UuidAndTitleCache<SpecimenOrObservationBase>> getUuidAndTitleCache() {
        return defaultService.getUuidAndTitleCache();
    }

    /**
     * @param clazz
     * @param limit
     * @param start
     * @param groups
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#group(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<Object[]> group(Class<? extends SpecimenOrObservationBase> clazz, Integer limit, Integer start, List<Grouping> groups, List<String> propertyPaths) {
        return defaultService.group(clazz, limit, start, groups, propertyPaths);
    }

    /**
     * @param occurence
     * @param taxonBase
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getDeterminations(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, eu.etaxonomy.cdm.model.taxon.TaxonBase, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<DeterminationEvent> getDeterminations(SpecimenOrObservationBase occurence, TaxonBase taxonBase, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getDeterminations(occurence, taxonBase, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<SpecimenOrObservationBase> findByTitle(Class<? extends SpecimenOrObservationBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.findByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param occurence
     * @param pageSize
     * @param pageNumber
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getDerivationEvents(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, java.lang.Integer, java.lang.Integer, java.util.List)
     */
    @Override
    public Pager<DerivationEvent> getDerivationEvents(SpecimenOrObservationBase occurence, Integer pageSize, Integer pageNumber, List<String> propertyPaths) {
        return defaultService.getDerivationEvents(occurence, pageSize, pageNumber, propertyPaths);
    }

    /**
     * @param type
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <S extends SpecimenOrObservationBase> List<S> list(Class<S> type, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.list(type, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param query
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#search(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<SpecimenOrObservationBase> search(Class<? extends SpecimenOrObservationBase> clazz, String query, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.search(clazz, query, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Pager<SpecimenOrObservationBase> findByTitle(IIdentifiableEntityServiceConfigurator<SpecimenOrObservationBase> configurator) {
        return defaultService.findByTitle(configurator);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List)
     */
    @Override
    public Integer countByTitle(Class<? extends SpecimenOrObservationBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria) {
        return defaultService.countByTitle(clazz, queryString, matchmode, criteria);
    }

    /**
     * @param uuid
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID)
     */
    @Override
    public SpecimenOrObservationBase load(UUID uuid) {
        return defaultService.load(uuid);
    }

    /**
     * @param uuid
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#load(java.util.UUID, java.util.List)
     */
    @Override
    public SpecimenOrObservationBase load(UUID uuid, List<String> propertyPaths) {
        return defaultService.load(uuid, propertyPaths);
    }

    /**
     * @param configurator
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#countByTitle(eu.etaxonomy.cdm.api.service.config.IIdentifiableEntityServiceConfigurator)
     */
    @Override
    public Integer countByTitle(IIdentifiableEntityServiceConfigurator<SpecimenOrObservationBase> configurator) {
        return defaultService.countByTitle(configurator);
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getFieldUnitUuidAndTitleCache()
     */
    @Override
    public List<UuidAndTitleCache<FieldUnit>> getFieldUnitUuidAndTitleCache() {
        return defaultService.getFieldUnitUuidAndTitleCache();
    }

    /**
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getDerivedUnitUuidAndTitleCache()
     */
    @Override
    public List<UuidAndTitleCache<DerivedUnit>> getDerivedUnitUuidAndTitleCache() {
        return defaultService.getDerivedUnitUuidAndTitleCache();
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#merge(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public SpecimenOrObservationBase merge(SpecimenOrObservationBase transientObject) {
        return defaultService.merge(transientObject);
    }

    /**
     * @param derivedUnit
     * @param propertyPaths
     * @return
     * @throws DerivedUnitFacadeNotSupportedException
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getDerivedUnitFacade(eu.etaxonomy.cdm.model.occurrence.DerivedUnit, java.util.List)
     */
    @Override
    public DerivedUnitFacade getDerivedUnitFacade(DerivedUnit derivedUnit, List<String> propertyPaths) throws DerivedUnitFacadeNotSupportedException {
        return defaultService.getDerivedUnitFacade(derivedUnit, propertyPaths);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<SpecimenOrObservationBase> listByTitle(Class<? extends SpecimenOrObservationBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.listByTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param description
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#listDerivedUnitFacades(eu.etaxonomy.cdm.model.description.DescriptionBase, java.util.List)
     */
    @Override
    public List<DerivedUnitFacade> listDerivedUnitFacades(DescriptionBase description, List<String> propertyPaths) {
        return defaultService.listDerivedUnitFacades(description, propertyPaths);
    }

    /**
     * @param type
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#page(java.lang.Class, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <S extends SpecimenOrObservationBase> Pager<S> page(Class<S> type, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.page(type, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param type
     * @param includeRelationships
     * @param associatedTaxon
     * @param maxDepth
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#listByAssociatedTaxon(java.lang.Class, java.util.Set, eu.etaxonomy.cdm.model.taxon.Taxon, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <T extends SpecimenOrObservationBase> List<T> listByAssociatedTaxon(Class<T> type, Set<TaxonRelationshipEdge> includeRelationships, Taxon associatedTaxon, Integer maxDepth, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.listByAssociatedTaxon(type, includeRelationships, associatedTaxon, maxDepth, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param persistentObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#refresh(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID refresh(SpecimenOrObservationBase persistentObject) {
        return defaultService.refresh(persistentObject);
    }

    /**
     * @param clazz
     * @param queryString
     * @param matchmode
     * @param criteria
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#listByReferenceTitle(java.lang.Class, java.lang.String, eu.etaxonomy.cdm.persistence.query.MatchMode, java.util.List, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<SpecimenOrObservationBase> listByReferenceTitle(Class<? extends SpecimenOrObservationBase> clazz, String queryString, MatchMode matchmode, List<Criterion> criteria, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.listByReferenceTitle(clazz, queryString, matchmode, criteria, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param tableName
     * @param limit
     * @param start
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#rows(java.lang.String, int, int)
     */
    @Override
    public List<SpecimenOrObservationBase> rows(String tableName, int limit, int start) {
        return defaultService.rows(tableName, limit, start);
    }

    /**
     * @param newInstances
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(java.util.Collection)
     */
    @Override
    public Map<UUID, SpecimenOrObservationBase> save(Collection<SpecimenOrObservationBase> newInstances) {
        for(SpecimenOrObservationBase unit:newInstances){
            save(unit);
        }
        return null;
    }

    /**
     * @param type
     * @param includeRelationships
     * @param associatedTaxon
     * @param maxDepth
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#pageByAssociatedTaxon(java.lang.Class, java.util.Set, eu.etaxonomy.cdm.model.taxon.Taxon, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <T extends SpecimenOrObservationBase> Pager<T> pageByAssociatedTaxon(Class<T> type, Set<TaxonRelationshipEdge> includeRelationships, Taxon associatedTaxon, Integer maxDepth, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.pageByAssociatedTaxon(type, includeRelationships, associatedTaxon, maxDepth, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param newInstance
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#save(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID save(SpecimenOrObservationBase newInstance) {
        units.add(newInstance);
        return null;
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID saveOrUpdate(SpecimenOrObservationBase transientObject) {
        units.add(transientObject);
        return null;
    }

    /**
     * @param transientObjects
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#saveOrUpdate(java.util.Collection)
     */
    @Override
    public Map<UUID, SpecimenOrObservationBase> saveOrUpdate(Collection<SpecimenOrObservationBase> transientObjects) {
        for(SpecimenOrObservationBase unit:transientObjects){
            saveOrUpdate(unit);
        }
        return null;
    }

    /**
     * @param clazz
     * @param queryString
     * @param boundingBox
     * @param languages
     * @param highlightFragments
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param propertyPaths
     * @return
     * @throws CorruptIndexException
     * @throws IOException
     * @throws ParseException
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#findByFullText(java.lang.Class, java.lang.String, org.hibernate.search.spatial.impl.Rectangle, java.util.List, boolean, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Pager<SearchResult<SpecimenOrObservationBase>> findByFullText(Class<? extends SpecimenOrObservationBase> clazz, String queryString, Rectangle boundingBox, List<Language> languages, boolean highlightFragments, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) throws CorruptIndexException, IOException, ParseException {
        return defaultService.findByFullText(clazz, queryString, boundingBox, languages, highlightFragments, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /**
     * @param transientObject
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#update(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public UUID update(SpecimenOrObservationBase transientObject) {
        return defaultService.update(transientObject);
    }

    /**
     * @param example
     * @param includeProperties
     * @param limit
     * @param start
     * @param orderHints
     * @param propertyPaths
     * @return
     * @see eu.etaxonomy.cdm.api.service.IService#list(eu.etaxonomy.cdm.model.common.ICdmBase, java.util.Set, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public List<SpecimenOrObservationBase> list(SpecimenOrObservationBase example, Set<String> includeProperties, Integer limit, Integer start, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.list(example, includeProperties, limit, start, orderHints, propertyPaths);
    }

    /**
     * @param clazz
     * @param matchStrategy
     * @param mergeStrategy
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#deduplicate(java.lang.Class, eu.etaxonomy.cdm.strategy.match.IMatchStrategy, eu.etaxonomy.cdm.strategy.merge.IMergeStrategy)
     */
    @Override
    public int deduplicate(Class<? extends SpecimenOrObservationBase> clazz, IMatchStrategy matchStrategy, IMergeStrategy mergeStrategy) {
        return defaultService.deduplicate(clazz, matchStrategy, mergeStrategy);
    }

    /**
     * @param clazz
     * @param queryString
     * @param pageSize
     * @param pageNumber
     * @param orderHints
     * @param matchMode
     * @return
     * @see eu.etaxonomy.cdm.api.service.IIdentifiableEntityService#findTitleCache(java.lang.Class, java.lang.String, java.lang.Integer, java.lang.Integer, java.util.List, eu.etaxonomy.cdm.persistence.query.MatchMode)
     */
    @Override
    public Pager<SpecimenOrObservationBase> findTitleCache(Class<? extends SpecimenOrObservationBase> clazz, String queryString, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints, MatchMode matchMode) {
        return defaultService.findTitleCache(clazz, queryString, pageSize, pageNumber, orderHints, matchMode);
    }

    /**
     * @return
     */
    public Collection<SpecimenOrObservationBase<?>> getUnits() {
        return units;
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getFieldUnits(java.util.UUID)
     */
    @Override
    public Collection<FieldUnit> getFieldUnits(UUID derivedUnitUuid) {
        return defaultService.getFieldUnits(derivedUnitUuid);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#pageByAssociatedTaxon(java.lang.Class, java.util.Set, java.lang.String, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public <T extends SpecimenOrObservationBase> Pager<T> pageByAssociatedTaxon(Class<T> type,
            Set<TaxonRelationshipEdge> includeRelationships, String taxonUUID, Integer maxDepth, Integer pageSize,
            Integer pageNumber, List<OrderHint> orderHints, List<String> propertyPaths) {
        return defaultService.pageByAssociatedTaxon(type, includeRelationships, taxonUUID,
                maxDepth, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#moveDerivate(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, eu.etaxonomy.cdm.model.occurrence.DerivedUnit)
     */
    @Override
    public boolean moveDerivate(SpecimenOrObservationBase<?> from, SpecimenOrObservationBase<?> to, DerivedUnit derivate) {
        return defaultService.moveDerivate(from, to, derivate);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#moveSequence(eu.etaxonomy.cdm.model.molecular.DnaSample, eu.etaxonomy.cdm.model.molecular.DnaSample, eu.etaxonomy.cdm.model.molecular.Sequence)
     */
    @Override
    public boolean moveSequence(DnaSample from, DnaSample to, Sequence sequence) {
        return defaultService.moveSequence(from, to, sequence);
    }

	@Override
	public List<String> isDeletable(SpecimenOrObservationBase object,
			DeleteConfiguratorBase config) {
		return defaultService.isDeletable(object, config);
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#countDeterminations(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, eu.etaxonomy.cdm.model.taxon.TaxonBase)
     */
    @Override
    public int countDeterminations(SpecimenOrObservationBase occurence, TaxonBase taxonbase) {
        return defaultService.countDeterminations(occurence, taxonbase);
    }

	@Override
	public SpecimenOrObservationBase findWithoutFlush(UUID uuid) {
		return defaultService.findWithoutFlush(uuid);
	}

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#listFieldUnitsByAssociatedTaxon(java.util.Set, eu.etaxonomy.cdm.model.taxon.Taxon, java.lang.Integer, java.lang.Integer, java.lang.Integer, java.util.List, java.util.List)
     */
    @Override
    public Collection<FieldUnit> listFieldUnitsByAssociatedTaxon(Set<TaxonRelationshipEdge> includeRelationships,
            Taxon associatedTaxon, Integer maxDepth, Integer pageSize, Integer pageNumber, List<OrderHint> orderHints,
            List<String> propertyPaths) {
        return defaultService.listFieldUnitsByAssociatedTaxon(includeRelationships, associatedTaxon, maxDepth, pageSize, pageNumber, orderHints, propertyPaths);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#assembleDerivateHierarchyDTO(eu.etaxonomy.cdm.model.occurrence.FieldUnit, java.util.UUID)
     */
    @Override
    public DerivateHierarchyDTO assembleDerivateHierarchyDTO(FieldUnit fieldUnit, UUID associatedTaxonUuid) {
        return defaultService.assembleDerivateHierarchyDTO(fieldUnit, associatedTaxonUuid);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#getNonCascadedAssociatedElements(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, eu.etaxonomy.cdm.api.conversation.ConversationHolder)
     */
    @Override
    public Collection<ICdmBase> getNonCascadedAssociatedElements(SpecimenOrObservationBase<?> specimen) {
        return defaultService.getNonCascadedAssociatedElements(specimen);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#deleteDerivateHierarchy(eu.etaxonomy.cdm.model.common.ICdmBase)
     */
    @Override
    public DeleteResult deleteDerivateHierarchy(ICdmBase from, SpecimenDeleteConfigurator config) {
        return defaultService.deleteDerivateHierarchy(from, config);
    }

    /* (non-Javadoc)
     * @see eu.etaxonomy.cdm.api.service.IOccurrenceService#delete(eu.etaxonomy.cdm.model.occurrence.SpecimenOrObservationBase, eu.etaxonomy.cdm.api.service.config.SpecimenDeleteConfigurator)
     */
    @Override
    public DeleteResult delete(SpecimenOrObservationBase<?> specimen, SpecimenDeleteConfigurator config) {
        return defaultService.delete(specimen, config);
    }

}
