/**
 * Copyright (C) 2007 EDIT
 * European Distributed Institute of Taxonomy
 * http://www.e-taxonomy.eu
 *
 * The contents of this file are subject to the Mozilla Public License Version 1.1
 * See LICENSE.TXT at the top of this package for the full license terms.
 */

package eu.etaxonomy.taxeditor.editor.view.derivate.operation;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.service.DeleteResult;
import eu.etaxonomy.cdm.api.service.IOccurrenceService;
import eu.etaxonomy.cdm.api.service.config.SpecimenDeleteConfigurator;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.model.MessagingUtils;
import eu.etaxonomy.taxeditor.operation.AbstractPostOperation;
import eu.etaxonomy.taxeditor.operation.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 *
 * @author pplitzner
 * @date Oct 21, 2014
 *
 */
public class DeleteDerivateOperation extends AbstractPostOperation<CdmBase> {

    private final SpecimenDeleteConfigurator deleteConfigurator;

    public DeleteDerivateOperation(String label, IUndoContext undoContext, CdmBase element,
            IPostOperationEnabled postOperationEnabled) {
        this(label, undoContext, element, postOperationEnabled, new SpecimenDeleteConfigurator());
    }

    public DeleteDerivateOperation(String label, IUndoContext undoContext, CdmBase element,
            IPostOperationEnabled postOperationEnabled, SpecimenDeleteConfigurator config) {
        super(label, undoContext, element, postOperationEnabled);
        this.deleteConfigurator = config;
    }


    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
     * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    /** {@inheritDoc} */
    @Override
    public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        DeleteResult deleteResult = CdmStore.getService(IOccurrenceService.class).deleteDerivateHierarchy(element, deleteConfigurator);
        if(!deleteResult.isOk()){
            String exceptionMessage = "";
            for(Exception exception:deleteResult.getExceptions()){
                exceptionMessage += exception.getLocalizedMessage();
            }
            //TODO: add method to DeleteResult to sum up exceptions
            MessagingUtils.warningDialog("Deletion failed", this, exceptionMessage);
            return postExecute(null);
        }
        return postExecute(element);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
     * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    /** {@inheritDoc} */
    @Override
    public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return execute(monitor, info);
    }

    /*
     * (non-Javadoc)
     *
     * @see
     * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
     * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
     */
    /** {@inheritDoc} */
    @Override
    public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
        return postExecute(element);
    }
}
